#include <GDD/ddg.h>
#include <SIRA/loop.h>

int main(int argc, char *argv[]) {
  // two operations
  GDD_Opcode *i1 =  GDD_opcode_create("inst_1", 1, 0);
  GDD_opcode_extend_delta_w(i1, "GR", 0);

  GDD_Opcode *i2 =  GDD_opcode_create("inst_2", 1, 0);
  GDD_opcode_extend_delta_w(i2, "GR", 0);

  GDD_Opcode *i3 =  GDD_opcode_create("inst_3", 1, 0);
  GDD_opcode_extend_delta_w(i3, "GR", 0);

  GDD_Opcode *i4 =  GDD_opcode_create("inst_4", 1, 0);
  GDD_opcode_extend_delta_w(i4, "GR", 0);

  GDD_Opcode *i5 =  GDD_opcode_create("inst_5", 1, 0);
  GDD_opcode_extend_delta_w(i5, "GR", 0);

  GDD_Opcode *i6 =  GDD_opcode_create("inst_6", 1, 0);
  GDD_opcode_extend_delta_w(i6, "BR", 0);

  GDD_Opcode *i7 =  GDD_opcode_create("inst_7", 1, 0);

  // build the architecture description
  GDD_Arch *arch = GDD_arch_create();
  GDD_arch_add_regtype(arch, "GR", 61);
  GDD_arch_add_regtype(arch, "BR", 8);
  GDD_arch_add_opcode(arch, i1);
  GDD_arch_add_opcode(arch, i2);
  GDD_arch_add_opcode(arch, i3);
  GDD_arch_add_opcode(arch, i4);
  GDD_arch_add_opcode(arch, i5);
  GDD_arch_add_opcode(arch, i6);
  GDD_arch_add_opcode(arch, i7);

  // build a DDG
  SCEDA_Graph *g = SCEDA_graph_create((SCEDA_delete_fun)GDD_node_label_delete, (SCEDA_delete_fun)GDD_edge_label_delete); 

  SCEDA_Vertex *v1 = SCEDA_graph_add_vertex(g, GDD_node_label_create(1, i1));
  SCEDA_Vertex *v2 = SCEDA_graph_add_vertex(g, GDD_node_label_create(2, i2));
  SCEDA_Vertex *v3 = SCEDA_graph_add_vertex(g, GDD_node_label_create(3, i3));
  SCEDA_Vertex *v4 = SCEDA_graph_add_vertex(g, GDD_node_label_create(4, i4));
  SCEDA_Vertex *v5 = SCEDA_graph_add_vertex(g, GDD_node_label_create(5, i5));
  SCEDA_Vertex *v6 = SCEDA_graph_add_vertex(g, GDD_node_label_create(6, i6));
  SCEDA_Vertex *v7 = SCEDA_graph_add_vertex(g, GDD_node_label_create(7, i7));

  SCEDA_graph_add_edge(g, v1, v2, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v1, v7, GDD_edge_label_create(NULL, "serial", 0, 0));
  SCEDA_graph_add_edge(g, v2, v3, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v3, v3, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v3, v7, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));      
  SCEDA_graph_add_edge(g, v4, v4, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v4, v6, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));
  SCEDA_graph_add_edge(g, v5, v1, GDD_edge_label_create(NULL, "serial", 1, 4));
  SCEDA_graph_add_edge(g, v5, v5, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v6, v7, GDD_edge_label_create("BR", "flowdep_reg", 3, 0));

  // create a siraloop with MIIrec = 2
  SIRA_Loop *loop = SIRA_loop_create(g, 2);

  // set sira method to compute max latency
  SIRA_loop_set_sira_method(loop);

  // for each II between IImin & IImax
  int II;
  for(II = SIRA_loop_IImin(loop); II <= SIRA_loop_IImax(loop); II++) {
    // get max latency
    int L = SIRA_loop_get_L(loop, II);

    fprintf(stdout,"II = %d, L = %d\n", II, L);

    // for each value of type GR
    SCEDA_HashSet *v_gr = GDD_get_values_of_type(g, "GR");

    SCEDA_HashSetIterator values;
    SCEDA_hashset_iterator_init(v_gr, &values);
    while(SCEDA_hashset_iterator_has_next(&values)) {
      SCEDA_Vertex *v = SCEDA_hashset_iterator_next(&values);
      // print the bounds for the schedule time of the value
      fprintf(stdout,"%d <= sigma(%s) <= %d\n", SIRA_loop_sigma_min(loop, v), GDD_vertex_mnemonic(v), SIRA_loop_sigma_max(loop, v, L));
      // and the bounds for its killer
      fprintf(stdout,"%d <= sigma_k(%s, GR) <= %d\n", SIRA_loop_k_min(loop, v, "GR", II), GDD_vertex_mnemonic(v), SIRA_loop_k_max(loop, v, L, "GR", II));
    }
    SCEDA_hashset_iterator_cleanup(&values);
    SCEDA_hashset_delete(v_gr);
  }

  SIRA_loop_delete(loop); // delete the loop

  SCEDA_graph_delete(g); // delete g
  GDD_arch_delete(arch);     // and the architecture

  return 0;
}
